#include <cstdlib>

#include "glew/glew.h"
#include "glut/freeglut.h"
#include "SPH.h"
#include "frame/frameRate.h"
#include "frame/frameGrab.h"

float zNear = 0.1f;
float zFar = 512.0f;
float fovy = 60.0f;

SPH* sph;

/* We use code from Chris Wyman (www.cs.uiowa.edu/~cwyman/) for calculating the fps and for dumping framebuffer to file */
FrameRate frameRate(30);
FrameGrab frameGrab("scene1");

GLfloat lightAmbient[] = { 0.5f, 0.5f, 0.5f, 1.0f };
GLfloat lightDiffuse[] = { 1.0f, 1.0f, 1.0f, 1.0f };
GLfloat lightPosition[]= { 1.0f, 1.0f, 1.0f, 0.0f };
GLfloat ballAmbientDiffuse[] = { 0.1f, 0.5f, 0.8f, 1.0f };
GLfloat ballSpecular[] = { 1.0f, 1.0f, 1.0f };
GLfloat ballShininess[] = { 50.0f };

void InitGL()
{  
  glEnable(GL_DEPTH_TEST);    
  glEnable(GL_BLEND);    
  glEnable(GL_LIGHTING);
  glEnable(GL_LIGHT0);
  glEnable(GL_LIGHT1);

  glClearColor(0.0f, 0.0f, 0.0f, 0.0f); 
  glDepthFunc(GL_LEQUAL);
  glShadeModel(GL_SMOOTH);
  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);  

  glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
  glHint(GL_LINE_SMOOTH_HINT, GL_NICEST);	
  glHint(GL_POLYGON_SMOOTH_HINT, GL_NICEST);

  glLightfv(GL_LIGHT0, GL_AMBIENT, lightAmbient);
  glLightfv(GL_LIGHT0, GL_DIFFUSE, lightDiffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);  

  glMaterialfv(GL_FRONT, GL_SPECULAR, ballSpecular);
  glMaterialfv(GL_FRONT, GL_SHININESS, ballShininess);
  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, ballAmbientDiffuse);
}

// Resizes the display window
void Reshape(int w, int h)
{  
  if (h == 0)
    h = 1;  
 
  glViewport(0, 0, w, h);

  glMatrixMode(GL_PROJECTION);
  glLoadIdentity();
  gluPerspective(fovy, (GLfloat) w / (GLfloat) h, zNear, zFar);
 
  glMatrixMode(GL_MODELVIEW);
  glLoadIdentity();
  gluLookAt(-50.0f, 60.0f, 50.0f, 0.0f, 0.0f, 0.0f, 0.0f, 1.0f, 0.0f);
}

// Main rendering callback
void Display()
{       
  frameRate.StartFrame();

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);    
  glPushMatrix();    
  sph->TimeStep();  
  glPopMatrix();
  glutSwapBuffers();

  frameRate.EndFrame();

  static int frameCount = 0;
  if (frameCount == 32)
  {
    printf("%.3f fps\n", frameRate.GetLastFrameRate());
    frameCount = 0;
  }  
  frameCount++;

  /*
  if (frameCount % 8 == 0)
  {
    frameGrab.CaptureFrame();
  }
  */
}

void Keyboard(unsigned char key, int x, int y)
{
  switch (key)
  {
  case '1':
    delete sph;
    sph = new SPH(1);
    break;
  case '2':
    delete sph;
    sph = new SPH(2);
    break;
  case '3':
    delete sph;
    sph = new SPH(3);
    break;
  case '4':
    delete sph;
    sph = new SPH(4);
    break;
  case '5':
    delete sph;
    sph = new SPH(5);
    break;
  case '6':
    delete sph;
    sph = new SPH(6);
    break;
  }
}

// zzz...
void Idle()
{
  glutPostRedisplay();
}

void CleanUp()
{
  delete sph;
}

int main(int argc, char* argv[])
{
  glewInit();

  glutInit(&argc, argv);
  glutSetOption(GLUT_ACTION_ON_WINDOW_CLOSE, GLUT_ACTION_GLUTMAINLOOP_RETURNS);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGBA | GLUT_DEPTH);
  glutInitWindowPosition(100, 100);
  glutInitWindowSize(640, 640);
  glutCreateWindow("SPH Madness"); 

  InitGL();    
    
  glutReshapeFunc(Reshape);
  glutDisplayFunc(Display);  
  glutIdleFunc(Idle);  
  glutKeyboardFunc(Keyboard);

  sph = new SPH(1);

  glutMainLoop();    

  CleanUp();
}
