#ifndef __SPH_H__
#define __SPH_H__

#include <vector>
#include "glut/freeglut.h"
#include "Eigen/Dense"
#include "Particle.h"
#include "Ultilities.h"

#define PI 3.14159265f

class SPH
{
public:
  typedef std::vector<Particle*> ParticleList;
  SPH(int sceneNumber);
  ~SPH();
  void TimeStep();
  EIGEN_MAKE_ALIGNED_OPERATOR_NEW

private:
  GLuint sphere_;
  GLuint ball_;

  unsigned int stepCount_;
  int sceneNumber_;
  float kernelParticles_;  
  float restDensity_;
  float pressureStiffness_;
  float viscosityCoefficient_;
  float gravitationalAcceleration_;
  float kernelRadius_;
  float kernelRadiusSquare_;
  float cellSize_;
  float timeStep_;
  float particleRadius_;
  float simulationScale_;  
  float wallStiffness_;
  float wallDamp_;
  float accelerationLimit_;
  unsigned int particleCount_;
  float particleMass_;

  Eigen::Vector4f minTotalVolume_;
  Eigen::Vector4f maxTotalVolume_;
  Eigen::Vector4f minGridVolume_;
  Eigen::Vector4f maxGridVolume_;
  unsigned int cellCount_;
  float realCellSize_;
  Eigen::Vector4f gridResolution_;    
  
  Particle* allParticles_; // all the particles
  ParticleList* neighbors_; // neighors[i] = a list of particle i's neighbors
  ParticleList* grid_; // data structure to speed up neighbor query  
  unsigned int* gridCount_;

  Eigen::Vector4f ballCenter_;
  float ballRadius_;
  float wallX_;

  /* Kernels */
  float LWViscosity; // laplacian of the WViscosity kernel
  float GWSpiky; // gradient of the WSpiky kernel
  float WPoly6; // WPoly6 kernel

  void ComputeKernels();
  void ComputePressure();
  void DetectCollisions(Particle& p);
  void InsertParticlesToVolume(unsigned int begin, const Eigen::Vector4f& min, const Eigen::Vector4f& max, float gap);
  void CreateVolumes();
  void CreateGrid(float border);
  void InsertParticlesToGrid();
  void InitializeParameters();
  void DrawParticles();
  void InitializeRendering();  
  void SortParticles();
  inline void ResponseToCollisions(const Eigen::Vector4f& velocity, float diff, const Eigen::Vector4f& wallNormal, Eigen::Vector4f& acceleration);
  unsigned int CountParticles(const Eigen::Vector4f& min, const Eigen::Vector4f& max, float gap);
  void DrawBall();
};

#endif